{% load paging_extras %}
{% load sentry_helpers %}

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"> 
<html lang="en">
    <head> 
        <meta http-equiv="content-type" content="text/html; charset=utf-8"> 
        <meta name="robots" content="NONE,NOARCHIVE">
        <link href="http{% if request.is_secure %}s{% endif %}://fonts.googleapis.com/css?family=Yanone+Kaffeesatz:light,regular&amp;subset=latin" rel="stylesheet" type="text/css">
        <link href="{% url sentry-media "styles/global.css" %}" rel="stylesheet" type="text/css"/>
        <link href="{% url sentry-media "images/sentry.png" %}" rel="shortcut icon"/>
        <title>{% block title %}Sentry{% endblock %}</title> 
        <script type="text/javascript" src="http{% if request.is_secure %}s{% endif %}://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js"></script>
        <script type="text/javascript" src="{% url sentry-media "scripts/highcharts.js" %}" ></script>
        <script type="text/javascript" src="{% url sentry-media "scripts/jquery.animate-colors-min.js" %}"></script>
        <script type="text/javascript" src="{% url sentry-media "scripts/global.js" %}"></script>
        <script type="text/javascript">
        Sentry.config({
            mediaUrl: '{% url sentry-media '' %}',
            apiUrl: '{% url sentry-ajax %}',
            defaultImage: '{% url sentry-media 'images/sentry.png' %}'
        });
        {% if has_realtime %}
        Sentry.realtime.enable();
        {% endif %}
        </script>
    </head> 
    
    <body> 
        {% block body %}
        <div id="header"> 
            <div class="container">
                <h1><a href="{% url sentry %}"><img src="{% url sentry-media "images/sentry.png" %}" border="0" width="32"/>Sentry</a></h1>
                <h2>{% block heading %}Dashboard{% endblock %}</h2>
                {% if request.user.is_authenticated %}
                    <form id="search" method="GET" action="{% url sentry-search %}">
                        <input type="hidden" name="sort" value="{% if sort %}{{ sort }}{% endif %}"/>
                        <input type="text" name="q" value="{% if query %}{{ query }}{% endif %}" placeholder="{% if has_search %}search query or message id{% else %}message id{% endif %}"/>
                        <button type="submit">Go</button>
                    </form>
                {% endif %}
            </div>
        </div> 
        <div id="content">
            {% block content_before %}
            {% endblock %}
            <div class="container">
                {% block content %}
                    <div id="sidebar" class="column">
                        {% block sidebar %}
                        {% endblock %}
                    </div>

                    <div id="body" class="with-sidebar column">
                        {% block main %}
                        {% endblock %}
                    </div>
                {% endblock %}
            </div>
         </div>
        <div id="footer">
            <div class="container">
                Sentry {% sentry_version %} | Powered by sexy Django magic | Conjured up by the <a href="http://code.disqus.com">DISQUS</a> team and other noble <a href="https://github.com/dcramer/django-sentry/contributors">sorcerers</a>.
                <ul class="nav" id="account">
                    {% if request.user.is_authenticated %}
                        <li><a href="{% url sentry-logout %}">Logout</a></li>
                    {% endif %}
                </ul>
            </div>
        </div> 
        {% endblock %}
    </body>
</html>
